# -*- coding: utf-8 -*-
"""
Created on Mon Sep 6 15:51:01 2021

@author: LuigiRussi
"""

#!/usr/bin/env python

## Post processing inspired from https://towardsdatascience.com/playing-with-time-series-data-in-python-959e2485bff8

import datetime
from datetime import date, time, timezone, timedelta
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.units as munits
from numpy.core.shape_base import block
converter = mdates.ConciseDateConverter()
munits.registry[np.datetime64] = converter
munits.registry[datetime.date] = converter
munits.registry[datetime.datetime] = converter
from scipy import integrate
from sklearn.linear_model import LinearRegression
import statsmodels.api as sm
import sys
np.set_printoptions(threshold=sys.maxsize)
import pandas as pd
from pandas.core import indexing
from pandas.core.indexes.base import Index
from pandas.tseries.offsets import Minute
import seaborn as sea
plt.style.use('seaborn-paper')
#sea.set_theme(color_codes=True)
import addcopyfighandler


def pp():

    # import data from NasoExt SD card, defined as a function in order to assign its output to a user defined variable


    def import_data_ext():
        raw_data_df = pd.read_csv(path_Ext, parse_dates=[
                              ['Date', 'Time']], sep=';', decimal=',', quotechar='"', engine='python', error_bad_lines=False)
        return raw_data_df


    TE = import_data_ext()
    TE.head()  # show the first 5 rows of the imported table

    # convert to numeric except datetime
    TE.rename({'Date_Time': 'DateTime'}, axis=1, inplace=True)
    TE[TE.columns[1:]].apply(pd.to_numeric, downcast='float', errors='coerce')

    TE['DateTime'] = TE['DateTime']+timedelta(hours=-1,minutes=9)
    TE = TE.set_index(TE.DateTime)

    # for some reason humidity is not converted, also lat and long have wrong separator
    # let's force conversion, check with TE.info()

    TE[TE.columns[1:]] = TE[TE.columns[1:]].astype('float')

    # set correct datetime type
    TE.DateTime = pd.to_datetime(TE.DateTime, errors='coerce')

    # index the series object with datetime column
    TE = TE.set_index(TE.DateTime)

    TE.head()

    # slice dataset
    TE = TE.loc[(TE.index > test_start) & (TE.index < test_end)]

    # use a rolling average for smoothing
    # 5 seconds for this dataset (add automation for sample rate retrievement)
    sample_rate = 5
    roll = int(60/sample_rate)
    TE_roll = TE.rolling(roll, center=True).mean()
    TE_std = TE.rolling(roll, center=True).std()

    # resample the smoothed dataset with minutely frequency
    TE_mm = TE_roll.resample('min').mean()
    TE_mm_std = TE_roll.resample('min').std()
    TE_mm = TE_mm.interpolate()

    # resample the smoothed dataset with hourly frequency
    TE_hh = TE_roll.resample('H').mean()
    TE_hh_std = TE_roll.resample('H').std()

    # import data from NasoInt SD card, defined as a function in order to assign its output to a user defined variable


    def import_data_int():
        raw_data_df = pd.read_csv(path_Int, parse_dates=[
                              ['Date', 'Time']], sep=',', decimal=',', quotechar='"', engine='python', error_bad_lines=False)
        return raw_data_df


    TI = import_data_int()
    TI.head()  # show the first 5 rows of the imported table

    # import DS sensorserial

    DSser = pd.read_csv('sensorserial.csv')
    #DSser.set_axis(['name_scheme','serial'],axis='index',inplace=True) # rename index
    DSser.rename({0:'position',1:'serial'},inplace=True)
    TI.rename({0:'serial'},inplace=True) # rename first line containing serials

    # connect naming scheme to serials in TI
    DSpos=pd.merge(DSser.T.astype(str),TI.T.astype(str),how='right',left_on='serial',right_on='serial')

    # rename variables accordingly
    DSpos['position'][23:43]
    newcolname=DSpos['position'][23:43]
    oldcolname=TI.columns[23:43]
    raw_dict=oldcolname.to_series().reset_index().drop(columns='index').join(newcolname.reset_index().drop(columns='index'))
    dict=raw_dict.set_index(0).to_dict()
    TI.rename(columns=dict['position'],inplace=True)
    TI.rename(columns={'Temp(C)':'FBC'},inplace=True)
    # clean dataset
    TI = TI.loc[:, TI.columns.notnull()] # drop nan column name
    TI.drop('serial',inplace=True) # drop serial row

    DS=['FBC','FBR','FMR','FTC','FTR','FBL','FML','FMC','FTL','BTR','BTC','BMR','BTL','BBR','BMC','BBC','BML','BBL']

    # convert to numeric except datetime
    TI.rename({'Date_Time': 'DateTime'}, axis=1, inplace=True)
    TI[TI.columns[1:]].apply(pd.to_numeric, downcast='float', errors='coerce')

    # set correct datetime type
    TI.DateTime = pd.to_datetime(TI.DateTime, errors='coerce')

    TI['DateTime'] = TI['DateTime']+timedelta(hours=0,minutes=0)
    TI = TI.set_index(TI.DateTime)

    # for some reason humidity is not converted, also lat and long have wrong separator
    # let's force conversion, check with TE.info()

    TI[TI.columns[1:]] = TI[TI.columns[1:]].astype('float')

    # set correct datetime type
    TI.DateTime = pd.to_datetime(TI.DateTime, errors='coerce')

    # index the series object with datetime column
    TI = TI.set_index(TI.DateTime)

    # compute mean over DS
    TI['T_avg']=TI[DS].mean(axis=1)

    TI.head()

    # slice dataset
    TI = TI.loc[(TI.index > test_start) & (TI.index < test_end)]

    # use a rolling average for smoothing
    # 5 seconds for this dataset (add automation for sample rate retrievement)
    sample_rate = 5
    roll = int(60/sample_rate)
    TI_roll = TI.rolling(roll, center=True).mean()
    TI_std = TI.rolling(roll, center=True).std()

    # resample the smoothed dataset with minutely frequency
    TI_mm = TI_roll.resample('min').mean()
    TI_mm_std = TI_roll.resample('min').std()
    TI_mm = TI_mm.interpolate()

    # resample the smoothed dataset with hourly frequency
    TI_hh = TI_roll.resample('H').mean()
    TI_hh_std = TI_roll.resample('H').std()

# import data from CANBUS, defined as a function in order to assign its output to a user defined variable


    def import_data_canbus():
        raw_data_df = pd.read_csv(path_Can, parse_dates=[
                              'DateTime'], sep=';', decimal='.', quotechar='"', engine='python', error_bad_lines=False)
        return raw_data_df

    TC = import_data_canbus()
    TC.head()  # show the first 5 rows of the imported table

    TC['DateTime'] = TC['DateTime']+timedelta(hours=1)
    TC = TC.set_index(TC.DateTime)

    # slice dataset
    TC = TC.loc[(TC.index > test_start) & (TC.index < test_end)]

    # use a rolling average for smoothing
    # 5 seconds for this dataset (add automation for sample rate retrievement)
    sample_rate = 5
    roll = int(60/sample_rate)
    TC_roll = TC.rolling(roll, center=True).mean()
    TC_std = TC.rolling(roll, center=True).std()

    # resample the smoothed dataset with minutely frequency
    TC_mm = TC_roll.resample('min').mean()
    TC_mm_std = TC_roll.resample('min').std()
    TC_mm = TC_mm.interpolate()


    # resample the smoothed dataset with hourly frequency
    TC_hh = TC_mm.resample('H').mean()
    TC_hh_std = TC_mm.resample('H').std()


    # merge the datasets

    #minutely
    TT = TE_mm.join(TI_mm, how='inner', lsuffix='_TE', rsuffix='_TI')
    TTT = TT.join(TC_mm, how='inner', lsuffix='', rsuffix='_TA')
    TTT = TTT.dropna()
    '''
    #hourly
    TT=TE_hh.join(TI_hh,how='inner',lsuffix='_TE',rsuffix='_TI')
    TTT=TT.join(TC_hh,how='inner',lsuffix='',rsuffix='_TA')
    '''


    ## PLOT SECTION ##

    # Set general plot appearance
    cm = 1/2.54  # centimeters in inches
    sea.set_theme(style='ticks', font_scale=1.5, rc={"lines.linewidth": 3})
    plt.rcParams["figure.figsize"] = (5*4*cm, 5*3*cm)
    plt.rcParams["figure.dpi"] = 150

    #Temperature

    sea.set_theme(style='ticks', font_scale=1.1, rc={"lines.linewidth": 3})   

    #multi-T
    fig, ax = plt.subplots(1,1,figsize=(5*4*cm,5*3*cm))
    
    # These are the colors that will be used in the plot
    ax.set_prop_cycle(color=[
    '#1f77b4', '#aec7e8', '#ff7f0e', '#ffbb78', '#2ca02c', '#98df8a',
    '#d62728', '#ff9896', '#9467bd', '#c5b0d5', '#8c564b', '#c49c94',
    '#e377c2', '#f7b6d2', '#7f7f7f', '#c7c7c7', '#bcbd22', '#dbdb8d',
    '#17becf', '#9edae5'])
    # Ensure that the axis ticks only show up on the bottom and left of the plot.
    # Ticks on the right and top of the plot are generally unnecessary.
    #ax.xaxis.tick_bottom()
    #ax.yaxis.tick_left()      

    # vertical line at AC on
    plt.axvline(x=[AC_on], color='black', label='A/C on', linestyle=':')

    TTT.plot(use_index=True,y=(DS),ax=ax,alpha=1)  

    # external temperature
    plt.plot(TTT.index,TTT['Temp(C)'], label='ext',color='black', linestyle='-')

    # Put the legend out of the figure
    plt.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
    ax.set_ylabel('Temperature (°C)')
    ax.set_xlabel('Time')
    plt.show()

 #multi-deltaT
    fig, ax = plt.subplots(1,1,figsize=(5*4*cm,5*3*cm))
    # These are the colors that will be used in the plot
    ax.set_prop_cycle(color=[
    '#1f77b4', '#aec7e8', '#ff7f0e', '#ffbb78', '#2ca02c', '#98df8a',
    '#d62728', '#ff9896', '#9467bd', '#c5b0d5', '#8c564b', '#c49c94',
    '#e377c2', '#f7b6d2', '#7f7f7f', '#c7c7c7', '#bcbd22', '#dbdb8d',
    '#17becf', '#9edae5'])
    # Ensure that the axis ticks only show up on the bottom and left of the plot.
    # Ticks on the right and top of the plot are generally unnecessary.
    #ax.xaxis.tick_bottom()
    #ax.yaxis.tick_left()

    # v line at AC on
    plt.axvline(x=[AC_on], color='black', label='A/C on', linestyle=':')

    (TTT[DS].sub(TTT['T_avg'],axis=0)).plot(use_index=True,y=DS,ax=ax,alpha=1)

    # h line at T_avg
    plt.plot(TTT.index,TTT['T_avg']-TTT['T_avg'], color='black', label=r'$T_{avg}$', linestyle='-')

    # Put the legend out of the figure
    plt.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
    ax.set_ylabel('Temperature difference (°C)')
    ax.set_xlabel('Time')
    plt.show()

    # reset theme

    sea.set_theme(style='ticks', font_scale=1.5, rc={"lines.linewidth": 3})   

    #inout-T
    fig, ax = plt.subplots()

    ax.set_ylabel('Temperature (°C)')
    ax.set_xlabel('Time')

    # vertical line at AC on
    plt.axvline(x=[AC_on], color='black', label='A/C on', linestyle=':')

    plt.plot(TTT.index, TTT['T_avg'], label='int')
    plt.plot(TTT.index, TTT['Temp(C)'], label='ext')
    plt.legend()

    #TVOC
    fig, ax = plt.subplots()
    ax.set_ylabel('TVOC (ppb)')
    ax.set_xlabel('Time')

    # vertical line at AC on
    plt.axvline(x=[AC_on], color='black', label='A/C on', linestyle=':')
    plt.plot(TTT.index, TTT['TVOC(ppb)_TI'], label='int')
    plt.plot(TTT.index, TTT['TVOC(ppb)_TE'], label='ext')
    plt.legend()

    #PM2.5
    fig, ax = plt.subplots()
    ax.set_ylabel('$PM2.5 (\mu g /m^3)$')
    ax.set_xlabel('Time')

    # vertical line at AC on
    plt.axvline(x=[AC_on], color='black', label='A/C on', linestyle=':')
    plt.plot(TTT.index, TTT['MC_PM2.5(ug/m3)_TI'], label='int')
    plt.plot(TTT.index, TTT['MC_PM2.5(ug/m3)_TE'], label='ext')
    plt.legend()

    #Power
    fig, ax = plt.subplots()
    ax.set_ylabel('Power usage (kW)')
    ax.set_xlabel('Time')

    # vertical line at AC on
    plt.axvline(x=[AC_on], color='black', linestyle=':', label='A/C on')
    plt.plot(TTT.index, (TTT['Aux_Pwr_100W']/1000),
            color='black', linestyle='-.', label='$P_{aux}$')
    plt.plot(TTT.index, (TTT['EstPwr_AC_50W']+TTT['Aux_Pwr_100W'])/1000,
            color='black', linestyle='--', label='$P_{aux}+P_{AC}$')
    plt.plot(TTT.index, (TTT['EstPwr_AC_50W']+TTT['Aux_Pwr_100W']
            +TTT['EstPwr_Htr_250W'])/1000, color='black', linestyle='-', label='$P_{aux}+P_{AC}+P_{htr}$')
    plt.legend()

    plt.ion()
    plt.show()
    #input("Press [enter] to continue.")
    return TTT


## Fresh-air old filter ##

# define test time
AC_on = pd.to_datetime('2021-01-29 13:25')
test_start = AC_on+timedelta(minutes=-15)
test_end = AC_on+timedelta(hours=1)
#duration = np.linspace(0,(test_end-test_start).total_seconds()/60,75)

# file locations
path_Ext="Test_29012021/log_nasocarext.CSV"
path_Int="Test_29012021/log_nasocarint1.CSV"
path_Can="Test_29012021/log_CAN.csv"

TTT_old_fresh = pp()

## Recirculation old filter ##

# define test time
AC_on = pd.to_datetime('2021-01-29 17:24')
test_start = AC_on+timedelta(minutes=-15)
test_end = AC_on+timedelta(hours=1)
#duration = np.linspace(0,(test_end-test_start).total_seconds()/60,75)

# file locations
path_Ext="Test_29012021/log_nasocarext.CSV"
path_Int="Test_29012021/log_nasocarint2.CSV"
path_Can="Test_29012021/log_CAN.csv"

TTT_old_rec = pp()

# Dimensionless and energy plots

t_set=30
t_star_fresh=(TTT_old_fresh['Temp(C)']-TTT_old_fresh['T_avg'])/(TTT_old_fresh['Temp(C)']-t_set)
t_star_rec=(TTT_old_rec['Temp(C)']-TTT_old_rec['T_avg'])/(TTT_old_rec['Temp(C)']-t_set)

fig, ax = plt.subplots()
ax.set_ylabel('Dimensionless temperature (t*)')
ax.set_xlabel('Time (min)')
# vertical line at AC on
plt.axvline(x=[3], color='black', linestyle=':', label='A/C on')
plt.plot(np.arange(0,np.size(t_star_fresh),1),t_star_fresh,color='black', linestyle='-',label='$t*_{fresh}$')
plt.plot(np.arange(0,np.size(t_star_rec),1),t_star_rec,color='black', linestyle='--',label='$t*_{rec}$')
plt.legend()
plt.ion()
plt.show()

fig, ax = plt.subplots()
ax.set_ylabel('PM2.5 filtration efficiency(%)')
ax.set_xlabel('Time (min)')
# vertical line at AC on
plt.axvline(x=[3], color='black', linestyle=':', label='A/C on')
plt.plot(np.arange(0,np.size(t_star_fresh),1),1-TTT_old_fresh['MC_PM2.5(ug/m3)_TI']/TTT_old_fresh['MC_PM2.5(ug/m3)_TE'],color='black', linestyle='-',label='$\eta_{fresh}$')
plt.plot(np.arange(0,np.size(t_star_rec),1),1-TTT_old_rec['MC_PM2.5(ug/m3)_TI']/TTT_old_rec['MC_PM2.5(ug/m3)_TE'],color='black', linestyle='--',label='$\eta_{rec}$')
plt.legend()
plt.ion()
plt.show()

fig, ax = plt.subplots()
ax.set_ylabel('TVOC filtration efficiency(%)')
ax.set_xlabel('Time (min)')
# vertical line at AC on
plt.axvline(x=[3], color='black', linestyle=':', label='A/C on')
plt.plot(np.arange(0,np.size(t_star_fresh),1),1-TTT_old_fresh['TVOC(ppb)_TI']/TTT_old_fresh['TVOC(ppb)_TE'],color='black', linestyle='-',label='$\eta_{fresh}$')
plt.plot(np.arange(0,np.size(t_star_rec),1),1-TTT_old_rec['TVOC(ppb)_TI']/TTT_old_rec['TVOC(ppb)_TE'],color='black', linestyle='--',label='$\eta_{rec}$')
plt.legend()
plt.ion()
plt.show()

# CAQI
caqi_fresh=(integrate.cumtrapz(TTT_old_fresh['MC_PM2.5(ug/m3)_TI']))/(integrate.cumtrapz(TTT_old_fresh['MC_PM2.5(ug/m3)_TE']))
caqi_rec=(integrate.cumtrapz(TTT_old_rec['MC_PM2.5(ug/m3)_TI']))/(integrate.cumtrapz(TTT_old_rec['MC_PM2.5(ug/m3)_TE']))

fig, ax = plt.subplots()
ax.set_ylabel('CAQI$_{PM2.5}$')
ax.set_xlabel('Time (min)')
# vertical line at AC on
plt.axvline(x=[3], color='black', linestyle=':', label='A/C on')
plt.plot(caqi_fresh,color='black', linestyle='-',label='$CAQI_{fresh}$')
plt.plot(caqi_rec,color='black', linestyle='--',label='$CAQI_{rec}$')
plt.legend()
plt.ion()
plt.show()

caqi_fresh_voc=(integrate.cumtrapz(TTT_old_fresh['TVOC(ppb)_TI']))/(integrate.cumtrapz(TTT_old_fresh['TVOC(ppb)_TE']))
caqi_rec_voc=(integrate.cumtrapz(TTT_old_rec['TVOC(ppb)_TI']))/(integrate.cumtrapz(TTT_old_rec['TVOC(ppb)_TE']))

fig, ax = plt.subplots()
ax.set_ylabel('CAQI$_{TVOC}$')
ax.set_xlabel('Time (min)')
# vertical line at AC on
plt.axvline(x=[3], color='black', linestyle=':', label='A/C on')
plt.plot(caqi_fresh_voc,color='black', linestyle='-',label='$CAQI_{fresh}$')
plt.plot(caqi_rec_voc,color='black', linestyle='--',label='$CAQI_{rec}$')
plt.legend()
plt.ion()
plt.show()

E_fresh = 1/60*integrate.trapz(TTT_old_fresh['EstPwr_AC_50W']+TTT_old_fresh['Aux_Pwr_100W']+TTT_old_fresh['EstPwr_Htr_250W'])/1000
E_rec = 1/60*integrate.trapz(TTT_old_rec['EstPwr_AC_50W']+TTT_old_rec['Aux_Pwr_100W']+TTT_old_rec['EstPwr_Htr_250W'])/1000
E_cum_fresh = 1/60*integrate.cumtrapz(TTT_old_fresh['EstPwr_AC_50W']+TTT_old_fresh['Aux_Pwr_100W']+TTT_old_fresh['EstPwr_Htr_250W'])/1000
E_cum_rec = 1/60*integrate.cumtrapz(TTT_old_rec['EstPwr_AC_50W']+TTT_old_rec['Aux_Pwr_100W']+TTT_old_rec['EstPwr_Htr_250W'])/1000

fig, ax = plt.subplots()
ax.set_ylabel('Cumulative energy use (kWh)')
ax.set_xlabel('Time (min)')
# vertical line at AC on
plt.axvline(x=[3], color='black', linestyle=':', label='A/C on')
plt.plot(E_cum_fresh,color='black', linestyle='-',label='$E_{fresh}=$'+f'{E_fresh:.2f}'+'$kWh$')
plt.plot(E_cum_rec,color='black', linestyle='--',label='$E_{rec}=$'+f'{E_rec:.2f}'+'$kWh$')
plt.legend()
plt.ion()
plt.show()

input("Press [enter] to continue.")
